# Generated by Django 5.2.1 on 2025-05-13 10:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0007_alter_notification_link'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='kind',
            field=models.CharField(choices=[('ERROR', 'Error'), ('SUCCESS', 'Success'), ('INFO', 'Info'), ('WARNING', 'Warning'), ('DANGER', 'Danger')], default='INFO', max_length=250, verbose_name='Kind'),
        ),
    ]
