# Generated by Django 5.1 on 2024-08-24 22:51
from django.conf import settings
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("notification", "0005_notification_expiry_date"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddIndex(
            model_name="notificationuser",
            index=models.Index(fields=["user"], name="notificatio_user_id_f8507b_idx"),
        ),
        migrations.AddIndex(
            model_name="notificationuser",
            index=models.Index(
                fields=["notification"], name="notificatio_notific_19025b_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="notificationuser",
            index=models.Index(fields=["seen"], name="notificatio_seen_0326fe_idx"),
        ),
    ]
