# Generated by Django 5.0.6 on 2024-07-02 13:39
from django.db import migrations
from django.db import models

import core.fields.image


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0015_alter_blogcategory_image"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogcategory",
            name="slug",
            field=models.SlugField(max_length=255, unique=True, verbose_name="Slug"),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="image",
            field=core.fields.image.ImageAndSvgField(
                blank=True, null=True, upload_to="uploads/blog/", verbose_name="Image"
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="slug",
            field=models.SlugField(max_length=255, unique=True, verbose_name="Slug"),
        ),
    ]
