# Generated by Django 5.2 on 2025-05-02 14:01

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cart', '0008_cart_session_key_cart_unique_session_cart'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cart',
            name='session_key',
            field=models.CharField(blank=True, default=uuid.uuid4, help_text='Session key for guest users', max_length=40, verbose_name='Session Key'),
        ),
    ]
