# Generated by Django 5.0.6 on 2024-05-13 13:43
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0004_settings_settings_value_type_idx"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Settings",
        ),
    ]
