# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contact', '0002_alter_contact_email_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='contact',
            name='contact_con_email_2ebc86_idx',
        ),
        migrations.AddIndex(
            model_name='contact',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['email'], name='contact_email_ix'),
        ),
    ]
