# Generated by Django 5.0.4 on 2024-04-13 18:00
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0005_alter_useraddress_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="useraddress",
            options={
                "ordering": ["-is_main", "-created_at"],
                "verbose_name": "User Address",
                "verbose_name_plural": "User Addresses",
            },
        ),
    ]
