# Generated by Django 5.2.1 on 2025-05-13 10:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0012_alter_orderhistory_user_agent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='floor',
            field=models.CharField(blank=True, choices=[('BASEMENT', 'Basement'), ('GROUND_FLOOR', 'Ground Floor'), ('FIRST_FLOOR', 'First Floor'), ('SECOND_FLOOR', 'Second Floor'), ('THIRD_FLOOR', 'Third Floor'), ('FOURTH_FLOOR', 'Fourth Floor'), ('FIFTH_FLOOR', 'Fifth Floor'), ('SIXTH_FLOOR_PLUS', 'Sixth Floor Plus')], default='', max_length=50, verbose_name='Floor'),
        ),
        migrations.AlterField(
            model_name='order',
            name='location_type',
            field=models.CharField(blank=True, choices=[('HOME', 'Home'), ('OFFICE', 'Office'), ('OTHER', 'Other')], default='', max_length=100, verbose_name='Location Type'),
        ),
    ]
