# Generated by Django 5.0.8 on 2024-08-17 16:22
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0015_alter_productfavourite_product_and_more"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="product",
            name="product_pro_final_p_0230ad_btree",
        ),
        migrations.RemoveIndex(
            model_name="product",
            name="product_pro_discoun_c1c630_btree",
        ),
        migrations.RemoveIndex(
            model_name="product",
            name="product_pro_price_s_7a19fc_btree",
        ),
        migrations.RemoveField(
            model_name="product",
            name="discount_value",
        ),
        migrations.RemoveField(
            model_name="product",
            name="discount_value_currency",
        ),
        migrations.RemoveField(
            model_name="product",
            name="final_price",
        ),
        migrations.RemoveField(
            model_name="product",
            name="final_price_currency",
        ),
        migrations.RemoveField(
            model_name="product",
            name="price_save_percent",
        ),
    ]
