# Generated by Django 5.2.5 on 2025-08-13 10:21

import user.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0017_alter_useraccount_facebook_alter_useraccount_github_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='useraccount',
            name='username',
            field=models.CharField(blank=True, error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 30 characters or fewer.Letters, digits and @/./+/-/_ only.', max_length=30, null=True, unique=True, validators=[user.validators.ExtendedUnicodeUsernameValidator()], verbose_name='Username'),
        ),
    ]
