# Generated by Django 5.2.6 on 2025-09-25 21:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0025_remove_product_product_product_code_ix_and_more'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='productcategory',
            index=models.Index(fields=['tree_id', 'lft'], name='product_productcategory_tr16fb'),
        ),
    ]
