# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('country', '0006_remove_country_country_cou_alpha_2_398383_idx_and_more'),
        ('region', '0005_alter_region_sort_order'),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='region',
            new_name='region_created_at_ix',
            old_name='region_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='region',
            new_name='region_updated_at_ix',
            old_name='region_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='region',
            new_name='region_sort_order_ix',
            old_name='region_sort_order_idx',
        ),
        migrations.AddIndex(
            model_name='region',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['alpha'], name='region_alpha_ix'),
        ),
        migrations.AddIndex(
            model_name='region',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['country'], name='region_country_ix'),
        ),
    ]
