# Generated by Django 5.2.1 on 2025-05-16 22:41

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pay_way', '0007_rename_payway_created_at_idx_payway_created_at_ix_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='payway',
            name='configuration',
            field=models.JSONField(blank=True, help_text='Provider-specific configuration (API keys, webhooks, etc.)', null=True, verbose_name='Provider Configuration'),
        ),
        migrations.AddField(
            model_name='payway',
            name='is_online_payment',
            field=models.BooleanField(default=False, help_text='Whether this payment method is processed online', verbose_name='Is Online Payment'),
        ),
        migrations.AddField(
            model_name='payway',
            name='provider_code',
            field=models.CharField(blank=True, default='', help_text="Code used to identify the payment provider in the system (e.g., 'stripe', 'paypal')", max_length=50, verbose_name='Provider Code'),
        ),
        migrations.AddField(
            model_name='payway',
            name='requires_confirmation',
            field=models.BooleanField(default=False, help_text='Whether this payment method requires manual confirmation (e.g., bank transfer)', verbose_name='Requires Confirmation'),
        ),
        migrations.AddField(
            model_name='paywaytranslation',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='paywaytranslation',
            name='instructions',
            field=models.TextField(blank=True, help_text='Instructions for the customer on how to complete payment (e.g., bank transfer details)', null=True, verbose_name='Payment Instructions'),
        ),
        migrations.AlterField(
            model_name='paywaytranslation',
            name='name',
            field=models.CharField(blank=True, choices=[('CREDIT_CARD', 'Credit Card'), ('PAY_ON_DELIVERY', 'Pay On Delivery'), ('PAY_ON_STORE', 'Pay On Store'), ('PAY_PAL', 'PayPal'), ('STRIPE', 'Stripe'), ('BANK_TRANSFER', 'Bank Transfer'), ('APPLE_PAY', 'Apple Pay'), ('GOOGLE_PAY', 'Google Pay')], max_length=50, null=True, verbose_name='Name'),
        ),
        migrations.AddIndex(
            model_name='payway',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['provider_code'], name='pay_way_provider_ix'),
        ),
        migrations.AddIndex(
            model_name='payway',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['is_online_payment'], name='pay_way_online_ix'),
        ),
    ]
