# Generated by Django 5.0.6 on 2024-05-16 11:58
import django.contrib.auth.validators
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0008_remove_useraccount_user_account_trgm_idx_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="useraccount",
            name="username",
            field=models.CharField(
                blank=True,
                error_messages={"unique": "A user with that username already exists."},
                help_text="Required. 30 characters or fewer.Letters, digits and @/./+/-/_ only.",
                max_length=30,
                null=True,
                unique=True,
                validators=[django.contrib.auth.validators.UnicodeUsernameValidator()],
                verbose_name="Username",
            ),
        ),
    ]
