# Generated by Django 5.2.6 on 2025-09-25 21:34

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0029_alter_blogcomment_approved'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddIndex(
            model_name='blogcategory',
            index=models.Index(fields=['tree_id', 'lft'], name='blog_blogcategory_tree_id_9462'),
        ),
        migrations.AddIndex(
            model_name='blogcomment',
            index=models.Index(fields=['tree_id', 'lft'], name='blog_blogcomment_tree_id_ld9de'),
        ),
    ]
