# Generated by Django 5.1 on 2024-08-24 22:45
import datetime

import django.db.models.expressions
import django.db.models.functions.datetime
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("notification", "0004_alter_notificationuser_notification"),
    ]

    operations = [
        migrations.AddField(
            model_name="notification",
            name="expiry_date",
            field=models.DateTimeField(
                blank=True,
                db_default=django.db.models.expressions.CombinedExpression(
                    django.db.models.functions.datetime.Now(),
                    "+",
                    models.Value(datetime.timedelta(days=180)),
                ),
                null=True,
                verbose_name="Expiry Date",
            ),
        ),
    ]
