# Generated by Django 5.2.5 on 2025-08-12 22:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0016_alter_subscriptiontopic_category_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='useraccount',
            name='facebook',
            field=models.URLField(blank=True, null=True, verbose_name='Facebook Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='github',
            field=models.URLField(blank=True, null=True, verbose_name='Github Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='instagram',
            field=models.URLField(blank=True, null=True, verbose_name='Instagram Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='linkedin',
            field=models.URLField(blank=True, null=True, verbose_name='LinkedIn Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='twitter',
            field=models.URLField(blank=True, null=True, verbose_name='Twitter Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='website',
            field=models.URLField(blank=True, null=True, verbose_name='Website'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='youtube',
            field=models.URLField(blank=True, null=True, verbose_name='Youtube Profile'),
        ),
    ]
