# Generated by Django 5.1.4 on 2024-12-31 00:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0008_alter_orderitem_sort_order'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='customer_notes',
            field=models.TextField(blank=True, default='', verbose_name='Customer Notes'),
        ),
        migrations.AlterField(
            model_name='order',
            name='floor',
            field=models.CharField(blank=True, choices=[(0, 'Basement'), (1, 'Ground Floor'), (2, 'First Floor'), (3, 'Second Floor'), (4, 'Third Floor'), (5, 'Fourth Floor'), (6, 'Fifth Floor'), (7, 'Sixth Floor Plus')], default='', max_length=50),
        ),
        migrations.AlterField(
            model_name='order',
            name='location_type',
            field=models.CharField(blank=True, choices=[(0, 'Home'), (1, 'Office'), (2, 'Other')], default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='order',
            name='place',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Place'),
        ),
    ]
