# Generated by Django 5.2.3 on 2025-06-21 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0010_notification_category_notification_notification_type_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='category',
            field=models.CharField(choices=[('ORDER', 'Order'), ('PAYMENT', 'Payment'), ('SHIPPING', 'Shipping'), ('CART', 'Cart'), ('PRODUCT', 'Product'), ('ACCOUNT', 'Account'), ('SECURITY', 'Security'), ('PROMOTION', 'Promotion'), ('SYSTEM', 'System'), ('REVIEW', 'Review'), ('WISHLIST', 'Wishlist'), ('SUPPORT', 'Support'), ('NEWSLETTER', 'Newsletter'), ('RECOMMENDATION', 'Recommendation')], default='SYSTEM', max_length=50, verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='priority',
            field=models.CharField(choices=[('LOW', 'Low Priority'), ('NORMAL', 'Normal Priority'), ('HIGH', 'High Priority'), ('URGENT', 'Urgent Priority'), ('CRITICAL', 'Critical Priority')], default='NORMAL', max_length=20, verbose_name='Priority'),
        ),
    ]
