# Generated by Django 5.2.3 on 2025-06-21 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0015_subscriptiontopic_subscriptiontopictranslation_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscriptiontopic',
            name='category',
            field=models.CharField(choices=[('MARKETING', 'Marketing Campaigns'), ('PRODUCT', 'Product Updates'), ('ACCOUNT', 'Account Updates'), ('SYSTEM', 'System Notifications'), ('NEWSLETTER', 'Newsletter'), ('PROMOTIONAL', 'Promotional'), ('OTHER', 'Other')], default='OTHER', help_text='Category of the subscription topic', max_length=20, verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='usersubscription',
            name='status',
            field=models.CharField(choices=[('ACTIVE', 'Active'), ('PENDING', 'Pending Confirmation'), ('UNSUBSCRIBED', 'Unsubscribed'), ('BOUNCED', 'Bounced')], default='ACTIVE', max_length=20, verbose_name='Status'),
        ),
    ]
