# Generated by Django 5.1 on 2024-09-01 21:58
import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0018_alter_productcategory_sort_order_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="producttranslation",
            options={
                "verbose_name": "Product Translation",
                "verbose_name_plural": "Product Translations",
            },
        ),
        migrations.RemoveField(
            model_name="producttranslation",
            name="search_document",
        ),
        migrations.RemoveField(
            model_name="producttranslation",
            name="search_document_dirty",
        ),
        migrations.RemoveField(
            model_name="producttranslation",
            name="search_vector",
        ),
        migrations.RemoveField(
            model_name="producttranslation",
            name="search_vector_dirty",
        ),
        migrations.AlterField(
            model_name="producttranslation",
            name="master",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="translations",
                to="product.product",
            ),
        ),
    ]
