# Generated by Django 5.2 on 2025-05-12 07:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0010_orderhistory_orderitemhistory_order_payment_id_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='orderhistory',
            name='user_agent',
            field=models.TextField(blank=True, help_text='User agent of the browser/client that made the change.', null=True, verbose_name='User Agent'),
        ),
    ]
