# Generated by Django 5.2.1 on 2025-05-15 18:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0025_rename_blogauthor_created_at_idx_blogauthor_created_at_ix_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='blogpost',
            name='blog_post_status_ix',
        ),
        migrations.RemoveIndex(
            model_name='blogpost',
            name='blog_post_status_featured_ix',
        ),
        migrations.RemoveField(
            model_name='blogpost',
            name='status',
        ),
    ]
