# Generated by Django 4.2.9 on 2024-02-09 23:38
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("core", "0001_pg_trigram_extension"),
    ]

    operations = [
        migrations.CreateModel(
            name="Settings",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "key",
                    models.CharField(max_length=255, unique=True, verbose_name="Key"),
                ),
                ("value", models.TextField(verbose_name="Value")),
                (
                    "value_type",
                    models.CharField(
                        choices=[
                            ("str", "String"),
                            ("int", "Integer"),
                            ("bool", "Boolean"),
                            ("dict", "Dictionary"),
                            ("list", "List"),
                            ("float", "Float"),
                        ],
                        default="str",
                        max_length=50,
                        verbose_name="Value Type",
                    ),
                ),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="Description"),
                ),
                (
                    "is_public",
                    models.BooleanField(default=True, verbose_name="Is Public"),
                ),
            ],
            options={
                "verbose_name": "Setting",
                "verbose_name_plural": "Settings",
            },
        ),
    ]
