# Generated by Django 5.0.3 on 2024-04-01 12:25
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0006_alter_blogpost_view_count"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="blogcomment",
            name="unique_blog_comment",
        ),
    ]
