# Generated by Django 5.0.3 on 2024-03-31 12:31
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0003_settings_site_alter_settings_unique_together"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="settings",
            index=models.Index(fields=["value_type"], name="settings_value_type_idx"),
        ),
    ]
