# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cart', '0009_alter_cart_session_key'),
        ('product', '0022_alter_productfavourite_options_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddIndex(
            model_name='cart',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['created_at'], name='cart_created_at_ix'),
        ),
        migrations.AddIndex(
            model_name='cart',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['updated_at'], name='cart_updated_at_ix'),
        ),
        migrations.AddIndex(
            model_name='cart',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['user'], name='cart_user_ix'),
        ),
        migrations.AddIndex(
            model_name='cart',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['session_key'], name='cart_session_key_ix'),
        ),
        migrations.AddIndex(
            model_name='cart',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['last_activity'], name='cart_last_activity_ix'),
        ),
        migrations.AddIndex(
            model_name='cartitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['created_at'], name='cartitem_created_at_ix'),
        ),
        migrations.AddIndex(
            model_name='cartitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['updated_at'], name='cartitem_updated_at_ix'),
        ),
        migrations.AddIndex(
            model_name='cartitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['cart'], name='cart_item_cart_ix'),
        ),
        migrations.AddIndex(
            model_name='cartitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['product'], name='cart_item_product_ix'),
        ),
        migrations.AddIndex(
            model_name='cartitem',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['quantity'], name='cart_item_quantity_ix'),
        ),
    ]
