# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0008_alter_notification_kind'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='notificationuser',
            name='notificatio_user_id_f8507b_idx',
        ),
        migrations.RemoveIndex(
            model_name='notificationuser',
            name='notificatio_notific_19025b_idx',
        ),
        migrations.RemoveIndex(
            model_name='notificationuser',
            name='notificatio_seen_0326fe_idx',
        ),
        migrations.RenameIndex(
            model_name='notification',
            new_name='notification_created_at_ix',
            old_name='notification_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='notification',
            new_name='notification_updated_at_ix',
            old_name='notification_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='notificationuser',
            new_name='notificationuser_created_at_ix',
            old_name='notificationuser_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='notificationuser',
            new_name='notificationuser_updated_at_ix',
            old_name='notificationuser_updated_at_idx',
        ),
        migrations.AddIndex(
            model_name='notification',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['kind'], name='notification_kind_ix'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['expiry_date'], name='notification_expiry_date_ix'),
        ),
        migrations.AddIndex(
            model_name='notificationuser',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['user'], name='not_user_user_ix'),
        ),
        migrations.AddIndex(
            model_name='notificationuser',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['notification'], name='not_user_notif_ix'),
        ),
        migrations.AddIndex(
            model_name='notificationuser',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['seen'], name='not_user_seen_ix'),
        ),
        migrations.AddIndex(
            model_name='notificationuser',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['seen_at'], name='not_user_seen_at_ix'),
        ),
    ]
