# Generated by Django 5.0.4 on 2024-04-11 11:43
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("pay_way", "0002_payway_payway_created_at_idx_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="payway",
            options={
                "ordering": ["-sort_order"],
                "verbose_name": "Pay Way",
                "verbose_name_plural": "Pay Ways",
            },
        ),
    ]
