# Generated by Django 5.0.3 on 2024-03-08 14:39
import measurement.measures.mass
from django.db import migrations

import core.fields.measurement
import core.weight


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0002_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="product",
            name="weight",
            field=core.fields.measurement.MeasurementField(
                default=core.weight.zero_weight,
                measurement=measurement.measures.mass.Mass,
                verbose_name="Weight",
            ),
        ),
    ]
