# Generated by Django 5.0.4 on 2024-04-11 13:34
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0008_alter_blogcategory_options_alter_blogtag_options"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="blogtag",
            managers=[],
        ),
    ]
