# Generated by Django 5.0.7 on 2024-07-21 00:11
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("tag", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="tag",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
            },
        ),
        migrations.AlterModelOptions(
            name="tagtranslation",
            options={
                "default_permissions": (),
                "managed": True,
                "verbose_name": "Tag Translation",
            },
        ),
    ]
